import tippy, { roundArrow } from "tippy.js";
import App from "./Translate.svelte";
import "./tippy.scss";
import { FrontendMessenger } from "../../background_frontend_commands";

(() => {
	//Make sure our script only runs once
	if (window.hasRun) return;
	window.hasRun = true;

	const con = new FrontendMessenger();

	const tippyInstance = setupTippy();

	let timeoutID: number;
	const onSelectionChange = () => {
		tippyInstance.hide();
		clearTimeout(timeoutID);
		if (document.getSelection()?.type == "Range")
			timeoutID = window.setTimeout(tippyInstance.show, 500);
	};

	con.addMessageListener("setEnabled", async (v: boolean) => {
		document.removeEventListener("selectionchange", onSelectionChange); //Always remove it avoid duplicate event listeners
		if (v) {
			onSelectionChange(); // Call it since selection may have changed since it was enabled
			document.addEventListener("selectionchange", onSelectionChange);
		} else tippyInstance.hide();
	});

	function setupTippy() {
		const svelteElement = document.createElement("div");
		const sveleteApp = new App({
			target: svelteElement,
			props: { toTranslate: "", connection: con },
		});
		return tippy(document.body, {
			content: svelteElement,
			trigger: "manual",
			interactive: true,
			arrow: roundArrow,
			theme: "webext",
			getReferenceClientRect: () =>
				document.getSelection()!.getRangeAt(0).getBoundingClientRect(),
			onCreate: (instance) => {
				sveleteApp.$on("translationFinished", () =>
					instance.popperInstance!.update()
				);
			},
			onShow: () => {
				sveleteApp.$set({
					toTranslate: document.getSelection()?.toString() ?? "",
				});
			},
		});
	}
})();
